#include <algorithm>
#include <cmath>
#include <msp/strings/format.h>
#include "libr2c2/tracktype.h"
#include "designer.h"
#include "manipulator.h"
#include "selection.h"

using namespace std;
using namespace R2C2;
using namespace Msp;

Manipulator::Manipulator(Designer &d, Input::Keyboard &k, Input::Mouse &m, const set<Object *> &objs):
	Tool(d, k, m)
{
	set<Object *> pending = objs;
	while(!pending.empty())
	{
		for(set<Object *>::iterator i=pending.begin(); i!=pending.end(); )
		{
			if((*i)->get_parent() && pending.count((*i)->get_parent()))
				++i;
			else
			{
				objects.push_back(*i);
				pending.erase(i++);
			}
		}
	}

	Geometry::BoundingBox<float, 3> bbox;
	for(vector<MObject>::iterator i=objects.begin(); i!=objects.end(); ++i)
		bbox = bbox|i->object->get_bounding_box();

	const Vector &minp = bbox.get_minimum_point();
	const Vector &maxp = bbox.get_maximum_point();

	center = (minp+maxp)/2.0f;
	center.z = minp.z;
	for(ObjectArray::iterator i=objects.begin(); i!=objects.end(); ++i)
	{
		i->original_position = i->object->get_position()-center;
		i->original_rotation = i->object->get_rotation();
	}
}

Manipulator::~Manipulator()
{
	if(!accepted)
	{
		for(ObjectArray::iterator i=objects.begin(); i!=objects.end(); ++i)
		{
			i->object->set_position(center+i->original_position);
			i->object->set_rotation(i->original_rotation);
		}
	}
}

void Manipulator::finish()
{
	set<Object *> object_set;
	for(ObjectArray::iterator i=objects.begin(); i!=objects.end(); ++i)
		object_set.insert(i->object);
	for(ObjectArray::iterator i=objects.begin(); i!=objects.end(); ++i)
	{
		unsigned nls = i->object->get_n_link_slots();
		for(unsigned j=0; j<nls; ++j)
			if(Object *link = i->object->get_link(j))
				if(!object_set.count(link))
					i->object->break_link(j);
	}

	const set<Track *> &ltracks = designer.get_layout().get_all<Track>();
	for(set<Track *>::const_iterator i=ltracks.begin(); i!=ltracks.end(); ++i)
	{
		bool ok = true;
		for(vector<MObject>::iterator j=objects.begin(); (j!=objects.end() && ok); ++j)
			ok = (j->object!=*i);
		if(!ok) continue;

		for(vector<MObject>::iterator j=objects.begin(); j!=objects.end(); ++j)
			j->object->link_to(**i);
	}
}


Manipulator::MObject::MObject(Object *o):
	object(o),
	original_position(object->get_position()),
	original_rotation(object->get_rotation())
{ }
