#include <msp/gl/meshbuilder.h>
#include <msp/gl/renderer.h>
#include <msp/gltk/graphic.h>
#include <msp/gltk/part.h>
#include "clockwidget.h"

using namespace std;
using namespace Msp;

ClockWidget::ClockWidget(R2C2::Clock &c):
	clock(c),
	hour_hand((GL::VERTEX2, GL::TEXCOORD2, GL::COLOR4_UBYTE)),
	minute_hand((GL::VERTEX2, GL::TEXCOORD2, GL::COLOR4_UBYTE))
{ }

void ClockWidget::rebuild_special(const GLtk::Part &part)
{
	if(part.get_name()=="hourhand" || part.get_name()=="minutehand")
	{
		bool minute = (part.get_name()=="minutehand");
		GL::Mesh &mesh = (minute ? minute_hand : hour_hand);

		const GLtk::Graphic *graphic = part.get_graphic(state);
		const GLtk::Alignment &align = part.get_alignment();

		mesh.clear();
		GL::MeshBuilder bld(mesh);
		bld.matrix() *= GL::Matrix::translation(GL::Vector3(graphic->get_width()*-align.x, graphic->get_height()*-align.y, 0));
		graphic->build(graphic->get_width(), graphic->get_height(), bld);

		part_cache.insert_special(part);
	}
}

void ClockWidget::render_special(const GLtk::Part &part, GL::Renderer &renderer) const
{
	if(part.get_name()=="hourhand" || part.get_name()=="minutehand")
	{
		bool minute = (part.get_name()=="minutehand");
		const GL::Mesh &mesh = (minute ? minute_hand : hour_hand);

		float orientation = clock.get_current_time()/((minute ? 1 : 12)*Time::hour);
		Geometry::Angle<float> angle = Geometry::Angle<float>::from_turns(orientation);

		GL::Renderer::Push push(renderer);
		renderer.matrix_stack() *= GL::Matrix::translation(GL::Vector3(geom.w*0.5f, geom.h*0.5f, 0.0f));
		renderer.matrix_stack() *= GL::Matrix::rotation(angle, GL::Vector3(0, 0, -1));
		mesh.draw(renderer);
	}
}
