#include "driver.h"
#include "layout.h"
#include "sensor.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Sensor::Sensor(Layout &l):
	layout(l),
	address(0),
	id(0),
	invert(false),
	state(INACTIVE)
{
	if(layout.has_driver())
		layout.get_driver().signal_sensor.connect(sigc::mem_fun(this, &Sensor::event));

	layout.add(*this);
}

Sensor::~Sensor()
{
	if(layout.has_driver() && address)
		layout.get_driver().remove_sensor(address);
	layout.remove(*this);
}

void Sensor::set_address(unsigned a)
{
	Driver *driver = (layout.has_driver() ? &layout.get_driver() : 0);
	if(driver && id)
		driver->remove_sensor(id);
	address = a;
	if(driver && address)
		id = driver->add_sensor(address);
	else
		id = 0;
}

void Sensor::tick(const Time::TimeDelta &dt)
{
	if(state_confirm_timeout)
	{
		state_confirm_timeout = max(state_confirm_timeout-dt, Time::zero);
		if(state_confirm_timeout<=Time::zero)
		{
			if(state==MAYBE_INACTIVE)
				state = INACTIVE;
			else if(state==MAYBE_ACTIVE)
				state = ACTIVE;
			signal_state_changed.emit(state);
		}
	}
}

void Sensor::event(unsigned i, bool s)
{
	if(i==id)
	{
		if(s!=invert && state<MAYBE_ACTIVE)
		{
			state = MAYBE_ACTIVE;
			state_confirm_timeout = 300*Time::msec;
			signal_state_changed.emit(state);
		}
		else if(s==invert && state>MAYBE_INACTIVE)
		{
			state = MAYBE_INACTIVE;
			state_confirm_timeout = 700*Time::msec;
			signal_state_changed.emit(state);
		}
	}
}

} // namespace R2C2
