#ifndef LIBR2C2_SIGNAL_H_
#define LIBR2C2_SIGNAL_H_

#include <msp/datafile/objectloader.h>
#include <msp/time/timedelta.h>
#include "blockiter.h"
#include "geometry.h"
#include "signaltype.h"
#include "trackattachment.h"

namespace R2C2 {

class Layout;
class Train;

class Signal: public TrackAttachment
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<Signal>
	{
	public:
		Loader(Signal &);

	private:
		void address(unsigned);
		void position(float, float, float);
		void rotation(float);
	};

private:
	const SignalType &type;
	unsigned address;
	unsigned id;
	BlockIter block;
	Train *train;
	sigc::connection train_conn;
	bool check_allocated_blocks;
	bool passing;

public:
	Signal(Layout &, const SignalType &);
	~Signal();

	virtual Signal *clone(Layout * = 0) const;
	virtual const SignalType &get_type() const { return type; }

	void set_address(unsigned);
	unsigned get_address() const { return address; }
	virtual void set_position(const Vector &);
	virtual void set_rotation(const Angle &);
	virtual void set_tilt(const Angle &) { }
private:
	void update_attachment();
public:

	virtual unsigned get_n_snap_nodes() const;
	virtual Snap get_snap_node(unsigned) const;
private:
	virtual SnapType get_default_snap_type_to(const Object &) const;

public:
	void tick(const Msp::Time::TimeDelta &);

private:
	void block_reserved(const Block &, Train *);
	void train_advanced(Block &);
	void reset();

public:
	void save(std::list<Msp::DataFile::Statement> &) const;
};
} // namespace R2C2


#endif
