#include <msp/strings/format.h>
#include "catalogue.h"
#include "layout.h"
#include "train.h"
#include "trainstatus.h"
#include "vehicle.h"

using namespace Msp;

namespace R2C2 {

TrainStatus::TrainStatus(Train &t):
	TrainAI(t),
	speed(-2)
{
	check();
}

void TrainStatus::tick(const Time::TimeDelta &)
{
	check();
}

void TrainStatus::check()
{
	float scale = train.get_layout().get_catalogue().get_scale();
	int s = static_cast<int>(train.get_quantized_speed()*3.6/scale+0.5);
	if(s==0 && train.get_block_allocator().is_active())
		s = -1;

	if(s!=speed)
	{
		if(s>0)
		{
			status = format("Traveling %d kmh", s);
			if(unsigned step = train.get_speed_step())
				status += format(" (%d)", step);
		}
		else if(s==-1)
			status = "Waiting";
		else if(!train.get_vehicle(0).is_placed())
			status = "Unplaced";
		else
			status = "Stopped";

		speed = s;

		signal_changed.emit(status);
		signal_event.emit(Message("status-changed", status));
	}
}

} // namespace R2C2
