#ifndef R2C2_NET_CLIENT_H_
#define R2C2_NET_CLIENT_H_

#include <msp/io/eventdispatcher.h>
#include <msp/net/communicator.h>
#include "libr2c2/catalogue.h"
#include "packets.h"
#include "protocol.h"
#include "train.h"

namespace R2C2 {

class Client: Msp::Net::PacketReceiver<DriverStatePacket>,
	Msp::Net::PacketReceiver<EmergencyPacket>,
	Msp::Net::PacketReceiver<TrainInfoPacket>,
	Msp::Net::PacketReceiver<TrainFunctionPacket>,
	Msp::Net::PacketReceiver<TrainControlPacket>,
	Msp::Net::PacketReceiver<TrainAIControlPacket>,
	Msp::Net::PacketReceiver<TrainStatusPacket>,
	Msp::Net::PacketReceiver<RouteInfoPacket>,
	Msp::Net::PacketReceiver<TrainRoutePacket>,
	Msp::Net::PacketReceiver<ErrorPacket>
{
public:
	sigc::signal<void, bool> signal_power_changed;
	sigc::signal<void, bool> signal_halt_changed;
	sigc::signal<void, const std::string &> signal_emergency;
	sigc::signal<void, NetTrain &> signal_train_added;
	sigc::signal<void, const std::string &> signal_error;

private:
	Catalogue &catalogue;
	Protocol proto;
	Msp::Net::StreamSocket *socket;
	Msp::Net::Communicator *comm;
	Msp::IO::EventDispatcher *event_disp;
	bool power;
	bool halt;
	std::list<std::string> routes;
	std::map<unsigned, NetTrain *> trains;

public:
	Client(Catalogue &);
	~Client();

	void use_event_dispatcher(Msp::IO::EventDispatcher &);
	void connect(const Msp::Net::SockAddr &);

	template<typename P>
	void send(const P &pkt)
	{ if(comm) comm->send(pkt); }

	Catalogue &get_catalogue() const { return catalogue; }
	const std::list<std::string> &get_routes() const { return routes; }
	void set_power(bool);
	void set_halt(bool);
	NetTrain &get_train(unsigned) const;
	const std::map<unsigned, NetTrain *> &get_trains() const { return trains; }

private:
	virtual void receive(const DriverStatePacket &);
	virtual void receive(const EmergencyPacket &);
	virtual void receive(const TrainInfoPacket &);
	virtual void receive(const TrainControlPacket &);
	virtual void receive(const TrainFunctionPacket &);
	virtual void receive(const TrainAIControlPacket &);
	virtual void receive(const TrainStatusPacket &);
	virtual void receive(const RouteInfoPacket &);
	virtual void receive(const TrainRoutePacket &);
	virtual void receive(const ErrorPacket &);
};

} // namespace R2C2

#endif
